import _regeneratorRuntime from 'babel-runtime/regenerator';
import _Promise from 'babel-runtime/core-js/promise';
import _slicedToArray from 'babel-runtime/helpers/slicedToArray';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _Object$getPrototypeOf from 'babel-runtime/core-js/object/get-prototype-of';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _ from 'lodash';

import * as arbitratorConstants from '../../constants/arbitrator';
import AbstractContract from '../AbstractContract';
import httpRequest from '../../utils/httpRequest';
import { DISPUTE_CACHE_URI } from '../../constants/dispute';
/**
 * Arbitrator Abstract Contarct API. This wraps an arbitrator contract. It provides
 * interaction with both the off chain store as well as the arbitrator instance. All
 * arbitrator methods from the supplied contract implementation can be called from this
 * object.
 */

var Arbitrator = function (_AbstractContract) {
  _inherits(Arbitrator, _AbstractContract);

  function Arbitrator() {
    var _ref,
        _this2 = this;

    var _temp, _this, _ret;

    _classCallCheck(this, Arbitrator);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = Arbitrator.__proto__ || _Object$getPrototypeOf(Arbitrator)).call.apply(_ref, [this].concat(args))), _this), _this.getDisputesForUser = function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(account) {
        var allowOffChainCache = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

        var _ref3, _ref4, period, currentSession, profile, cachedDisputes, cachedDisputesResponse, myDisputes;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return _Promise.all([_this._contractImplementation.getPeriod(), _this._contractImplementation.getSession()]);

              case 2:
                _ref3 = _context2.sent;
                _ref4 = _slicedToArray(_ref3, 2);
                period = _ref4[0];
                currentSession = _ref4[1];

                if (!(period !== arbitratorConstants.PERIOD.VOTE)) {
                  _context2.next = 8;
                  break;
                }

                return _context2.abrupt('return', _this.getDisputesForUserFromStore(account));

              case 8:
                _context2.next = 10;
                return _this._StoreProvider.newUserProfile(account);

              case 10:
                profile = _context2.sent;

                if (!(currentSession !== profile.session)) {
                  _context2.next = 27;
                  break;
                }

                // pull open disputes from off chain cache
                cachedDisputes = null;

                if (!allowOffChainCache) {
                  _context2.next = 21;
                  break;
                }

                _context2.next = 16;
                return httpRequest('GET', DISPUTE_CACHE_URI + '/' + currentSession);

              case 16:
                cachedDisputesResponse = _context2.sent;

                if (!(cachedDisputesResponse.body && cachedDisputesResponse.body.open_disputes)) {
                  _context2.next = 21;
                  break;
                }

                _context2.next = 20;
                return _Promise.all(cachedDisputesResponse.body.open_disputes.map(function (disputeIDString) {
                  return _this._contractImplementation.getDispute(Number(disputeIDString));
                }));

              case 20:
                cachedDisputes = _context2.sent;

              case 21:
                _context2.next = 23;
                return _this._contractImplementation.getDisputesForJuror(account, cachedDisputes);

              case 23:
                myDisputes = _context2.sent;
                _context2.next = 26;
                return _Promise.all(myDisputes.map(function () {
                  var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispute) {
                    var disputeCreationLog;
                    return _regeneratorRuntime.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            if (!(dispute.appealDraws[dispute.numberOfAppeals].length > 0)) {
                              _context.next = 10;
                              break;
                            }

                            _context.next = 3;
                            return _this._contractImplementation.getDisputeCreationEvent(dispute.disputeID);

                          case 3:
                            disputeCreationLog = _context.sent;

                            if (disputeCreationLog) {
                              _context.next = 6;
                              break;
                            }

                            throw new Error('Could not fetch dispute creation event log');

                          case 6:
                            _context.next = 8;
                            return _this._StoreProvider.updateDisputeProfile(account, dispute.arbitratorAddress, dispute.disputeID, {
                              blockNumber: disputeCreationLog.blockNumber
                            });

                          case 8:
                            _context.next = 10;
                            return _this._StoreProvider.addNewDrawsDisputeProfile(account, dispute.arbitratorAddress, dispute.disputeID, dispute.appealDraws[dispute.numberOfAppeals], dispute.numberOfAppeals);

                          case 10:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, _this2);
                  }));

                  return function (_x3) {
                    return _ref5.apply(this, arguments);
                  };
                }()));

              case 26:

                // FIXME do we want to store session?
                _this._StoreProvider.updateUserSession(account, currentSession);

              case 27:
                return _context2.abrupt('return', _this.getDisputesForUserFromStore(account));

              case 28:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this2);
      }));

      return function (_x) {
        return _ref2.apply(this, arguments);
      };
    }(), _this.getDisputesForUserFromStore = function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(account) {
        var aribtratorAddress;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                aribtratorAddress = _this._contractImplementation.getContractAddress();
                _context3.t0 = _Promise;
                _context3.next = 4;
                return _this._StoreProvider.getDisputes(account);

              case 4:
                _context3.t1 = function (dispute) {
                  return dispute.arbitratorAddress === aribtratorAddress;
                };

                _context3.t2 = function (dispute) {
                  return _this._contractImplementation.getDispute(dispute.disputeId);
                };

                _context3.t3 = _context3.sent.filter(_context3.t1).map(_context3.t2);
                return _context3.abrupt('return', _context3.t0.all.call(_context3.t0, _context3.t3));

              case 8:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this2);
      }));

      return function (_x4) {
        return _ref6.apply(this, arguments);
      };
    }(), _this.buyPNK = function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(amount, account) {
        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return _this._contractImplementation.buyPNK(amount, account);

              case 2:
                return _context4.abrupt('return', _this._contractImplementation.getPNKBalance(account));

              case 3:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, _this2);
      }));

      return function (_x5, _x6) {
        return _ref7.apply(this, arguments);
      };
    }(), _temp), _possibleConstructorReturn(_this, _ret);
  }
  /**
   * Get disputes for user with extra data from arbitrated transaction and store
   * @param {string} account Address of user
   * @param {bool} allowOffChainCache Should open disputes be pulled from off chain cache.
   * The cache is maintained by Kleros for performance. To pull open disputes from the blockchain
   * set false
   * @returns {object[]} dispute data objects for user
   */


  /**
   * Get disputes from the store.
   * @param {string} account - The users account.
   * @returns {object[]} The dispute objects.
   */


  /**
   * Buy PNK.
   * @param {number} amount - Number of pinakion to buy.
   * @param {string} account - Address of user.
   * @returns {object[]} - Balance of user.
   */


  return Arbitrator;
}(AbstractContract);

export default Arbitrator;
//# sourceMappingURL=Arbitrator.js.map