import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _Object$getPrototypeOf from 'babel-runtime/core-js/object/get-prototype-of';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';

var _this2 = this;

import klerosPOCArtifact from 'kleros/build/contracts/KlerosPOC';
import _ from 'lodash';

import * as ethConstants from '../../../constants/eth';
import deployContractAsync from '../../../utils/deployContractAsync';

import Kleros from './Kleros';

/**
 * Provides interaction with a KlerosPOC contract on the blockchain.
 */

var KlerosPOC = function (_Kleros) {
  _inherits(KlerosPOC, _Kleros);

  /**
   * Create new KlerosPOC Implementation.
   * @param {object} web3Provider - web3 instance.
   * @param {string} contractAddress - Address of the KlerosPOC contract.
   */
  function KlerosPOC(web3Provider, contractAddress) {
    _classCallCheck(this, KlerosPOC);

    var _this = _possibleConstructorReturn(this, (KlerosPOC.__proto__ || _Object$getPrototypeOf(KlerosPOC)).call(this, web3Provider, contractAddress, klerosPOCArtifact));

    _initialiseProps.call(_this);

    return _this;
  }

  /**
   * STATIC: Deploy a KlerosPOC contract on the blockchain.
   * @param {string} rngAddress address of random number generator contract
   * @param {string} pnkAddress address of pinakion contract
   * @param {number[]} timesPerPeriod array of 5 ints indicating the time limit for each period of contract
   * @param {string} account address of user
   * @param {number} value amout of eth to send to contract
   * @param {object} web3Provider web3 provider object NOTE: NOT Kleros Web3Wrapper
   * @returns {object} truffle-contract Object | err The contract object or error deploy
   */


  /**
   * Purchase PNK.
   * @param {string} amount - The amount of pinakion to buy in wei.
   * @param {string} account - The address of the user.
   * @returns {object} - The result transaction object.
   */


  return KlerosPOC;
}(Kleros);

KlerosPOC.deploy = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(rngAddress, pnkAddress) {
    var timesPerPeriod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [300, 0, 300, 300, 300];
    var account = arguments[3];
    var value = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : ethConstants.TRANSACTION.VALUE;
    var web3Provider = arguments[5];
    var contractDeployed;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return deployContractAsync(account, value, klerosPOCArtifact, web3Provider, pnkAddress, rngAddress, timesPerPeriod);

          case 2:
            contractDeployed = _context.sent;
            return _context.abrupt('return', contractDeployed);

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this2);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var _initialiseProps = function _initialiseProps() {
  var _this3 = this;

  this.buyPNK = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(amount, account) {
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return _this3.loadContract();

            case 2:
              return _context2.abrupt('return', _this3.contractInstance.buyPinakion({
                from: account,
                value: amount,
                gas: process.env.GAS || undefined
              }));

            case 3:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this3);
    }));

    return function (_x5, _x6) {
      return _ref2.apply(this, arguments);
    };
  }();
};

export default KlerosPOC;
//# sourceMappingURL=KlerosPOC.js.map