'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ContractImplementation = require('../../ContractImplementation');

var _ContractImplementation2 = _interopRequireDefault(_ContractImplementation);

var _EventListener = require('../../../utils/EventListener');

var _EventListener2 = _interopRequireDefault(_EventListener);

var _httpRequest = require('../../../utils/httpRequest');

var _httpRequest2 = _interopRequireDefault(_httpRequest);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides interaction with standard Arbitrable contracts
 */
var Arbitrable = function (_ContractImplementati) {
  (0, _inherits3.default)(Arbitrable, _ContractImplementati);

  /**
   * Constructor ArbitrableTransaction.
   * @param {object} web3Provider instance.
   * @param {object} contractArtifact The abi JSON of the arbitrable contract.
   * @param {string} contractAddress of the contract.
   */
  function Arbitrable(web3Provider, contractArtifact, contractAddress) {
    var _this2 = this;

    (0, _classCallCheck3.default)(this, Arbitrable);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Arbitrable.__proto__ || (0, _getPrototypeOf2.default)(Arbitrable)).call(this, web3Provider, contractArtifact, contractAddress));

    _this.getMetaEvidence = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var metaEvidenceID = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
        var metaEvidenceLog, metaEvidenceUri, metaEvidenceResponse;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!_this.metaEvidenceCache[metaEvidenceID]) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', _this.metaEvidenceCache[metaEvidenceID]);

              case 2:
                _context.next = 4;
                return _EventListener2.default.getEventLogs(_this, 'MetaEvidence', 0, 'latest', { _metaEvidenceID: metaEvidenceID });

              case 4:
                metaEvidenceLog = _context.sent;

                if (metaEvidenceLog[0]) {
                  _context.next = 7;
                  break;
                }

                return _context.abrupt('return', {});

              case 7:
                // NOTE better to throw errors for missing meta-evidence?

                metaEvidenceUri = metaEvidenceLog[0].args._evidence;
                // FIXME caching issue need a query param to fetch from AWS

                _context.next = 10;
                return (0, _httpRequest2.default)('GET', metaEvidenceUri);

              case 10:
                metaEvidenceResponse = _context.sent;

                if (!(metaEvidenceResponse.status >= 400)) {
                  _context.next = 13;
                  break;
                }

                throw new Error('Unable to fetch meta-evidence at ' + metaEvidenceUri);

              case 13:

                _this.metaEvidenceCache[metaEvidenceID] = metaEvidenceResponse.body || metaEvidenceResponse;
                return _context.abrupt('return', metaEvidenceResponse.body || metaEvidenceResponse);

              case 15:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this2);
      }));

      return function () {
        return _ref.apply(this, arguments);
      };
    }();

    _this.getEvidence = function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(arbitratorAddress) {
        var disputeID = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var evidenceLogs;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return _EventListener2.default.getEventLogs(_this, 'Evidence', 0, 'latest', { _disputeID: disputeID, _arbitrator: arbitratorAddress });

              case 2:
                evidenceLogs = _context3.sent;
                return _context3.abrupt('return', _promise2.default.all(evidenceLogs.map(function () {
                  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(evidenceLog) {
                    var evidenceURI, evidence, submittedAt;
                    return _regenerator2.default.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            evidenceURI = evidenceLog.args._evidence;
                            _context2.next = 3;
                            return (0, _httpRequest2.default)('GET', evidenceURI);

                          case 3:
                            evidence = _context2.sent;
                            _context2.next = 6;
                            return _this._Web3Wrapper.getBlock(evidenceLog.blockNumber);

                          case 6:
                            submittedAt = _context2.sent.timestamp;
                            return _context2.abrupt('return', (0, _extends3.default)({}, evidence.body, { submittedBy: evidenceLog.args._party, submittedAt: submittedAt }));

                          case 8:
                          case 'end':
                            return _context2.stop();
                        }
                      }
                    }, _callee2, _this2);
                  }));

                  return function (_x4) {
                    return _ref3.apply(this, arguments);
                  };
                }())));

              case 4:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this2);
      }));

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    }();

    _this.getContractData = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
      var _ref5, _ref6, metaEvidence;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return _this.loadContract();

            case 2:
              _context4.next = 4;
              return _promise2.default.all([_this.getMetaEvidence()]);

            case 4:
              _ref5 = _context4.sent;
              _ref6 = (0, _slicedToArray3.default)(_ref5, 1);
              metaEvidence = _ref6[0];
              return _context4.abrupt('return', {
                metaEvidence: metaEvidence
              });

            case 8:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, _this2);
    }));


    _this.metaEvidenceCache = {};
    return _this;
  }

  /**
   * Get the meta evidence for the contract. Arbitrable Transaction can only have
   * one meta-evidence that is submitted on contract creation. Look up meta-evidence event
   * and make an http request to the resource.
   * @returns {object} The metaEvidence object
   */


  /**
   * Get the evidence submitted in a dispute.
   * @returns {object[]} An array of evidence objects.
   */


  /**
   * Fetch all standard contract data.
   */


  return Arbitrable;
}(_ContractImplementation2.default);

exports.default = Arbitrable;
//# sourceMappingURL=Arbitrable.js.map