'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _trunc = require('babel-runtime/core-js/math/trunc');

var _trunc2 = _interopRequireDefault(_trunc);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _MultipleArbitrableTransaction = require('kleros-interaction/build/contracts/MultipleArbitrableTransaction');

var _MultipleArbitrableTransaction2 = _interopRequireDefault(_MultipleArbitrableTransaction);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _contract = require('../../../constants/contract');

var contractConstants = _interopRequireWildcard(_contract);

var _error = require('../../../constants/error');

var errorConstants = _interopRequireWildcard(_error);

var _deployContractAsync = require('../../../utils/deployContractAsync');

var _deployContractAsync2 = _interopRequireDefault(_deployContractAsync);

var _Arbitrable2 = require('./Arbitrable');

var _Arbitrable3 = _interopRequireDefault(_Arbitrable2);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides interaction with an Arbitrable Transaction contract deployed on the blockchain.
 */
var MultipleArbitrableTransaction = function (_Arbitrable) {
  (0, _inherits3.default)(MultipleArbitrableTransaction, _Arbitrable);

  /**
   * Constructor ArbitrableTransaction.
   * @param {object} web3Provider instance
   * @param {string} contractAddress of the contract
   */
  function MultipleArbitrableTransaction(web3Provider, contractAddress) {
    (0, _classCallCheck3.default)(this, MultipleArbitrableTransaction);

    var _this = (0, _possibleConstructorReturn3.default)(this, (MultipleArbitrableTransaction.__proto__ || (0, _getPrototypeOf2.default)(MultipleArbitrableTransaction)).call(this, web3Provider, _MultipleArbitrableTransaction2.default, contractAddress));

    _initialiseProps.call(_this);

    _this.arbitrableTransactionId = null;
    return _this;
  }

  /**
   * Deploy MultipleArbitrableTransaction.
   * @param {object} account Ethereum account
   * @param {object} web3Provider web3 provider object
   * @returns {object} truffle-contract Object | err The deployed contract or an error
   */


  /**
   * Create MultipleArbitrableTransaction.
   * @param {object} account Ethereum account
   * @param {string} arbitratorAddress The address of the arbitrator contract
   * @param {object} seller Seller Ethereum account
   * @param {number} value funds to be placed in contract
   * @param {number} timeout Time (seconds) after which a party automatically loose a dispute. (default 3600)
   * @param {bytes} arbitratorExtraData Extra data for the arbitrator. (default empty string)
   * @param {string} metaEvidenceUri Uri meta-evidence. (default empty string)
   * @returns {object} truffle-contract Object | err The deployed contract or an error
   */


  /**
   * Pay the seller. To be called when the good is delivered or the service rendered.
   * @param {string} account - Ethereum account.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {amount} amount - Part or all of the amount of the good or the service.
   * @returns {object} - The result transaction object.
   */


  /**
   * Reimburse the seller. To be called when the good is not delivered or the service rendered.
   * @param {string} account - Ethereum account.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {amount} amount - Part or all of the amount of the good or the service.
   * @returns {object} - The result transaction object.
   */


  /**
   * Pay the arbitration fee to raise a dispute. To be called by the buyer.
   * @param {string} account - Ethereum account.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {number} arbitrationCost - Arbitration cost.
   * @returns {object} - The result transaction object.
   */


  /**
   * Pay the arbitration fee to raise a dispute. To be called by the seller.
   * @param {string} account Ethereum account.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {number} arbitrationCost - Arbitration cost.
   * @returns {object} - The result transaction object.
   */


  /**
   * Submit evidence.
   * @param {string} account ETH address of user.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {string} url A link to an evidence using its URI.
   * @returns {string} txHash Hash transaction.
   */


  /**
   * Call by buyer if seller is timeout
   * @param {string} account ETH address of user
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @returns {object} The result transaction object.
   */


  /**
   * Call by seller if buyer is timeout.
   * @param {string} account - ETH address of user.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {string} contractAddress - ETH address of contract.
   * @returns {object} The result transaction object.
   */


  /**
   * Appeal an appealable ruling.
   * @param {string} account Ethereum account.
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @param {bytes} extraData for the arbitrator appeal procedure.
   * @param {number} appealCost Amount to pay the arbitrator. (default 0.35 ether).
   * @returns {object} - The result transaction object.
   */


  /**
   * Set the arbitrable transaction id
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @returns {object} Object Data of the contract.
   */


  /**
   * Fetch the parties involved in the arbitrable transaction contract.
   * @returns {object} returns a mapping of partyA and partyB to ETH addresses.
   */


  /**
   * Data of the contract
   * @param {number} arbitrableTransactionId - The index of the transaction.
   * @returns {object} Object Data of the contract.
   */


  return MultipleArbitrableTransaction;
}(_Arbitrable3.default);

MultipleArbitrableTransaction.deploy = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(account, web3Provider) {
    var contractDeployed;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _deployContractAsync2.default)(account, 0, _MultipleArbitrableTransaction2.default, web3Provider);

          case 2:
            contractDeployed = _context.sent;
            return _context.abrupt('return', contractDeployed);

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.createArbitrableTransaction = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(account, arbitratorAddress, seller, value) {
      var timeout = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 3600;
      var arbitratorExtraData = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0x0;
      var metaEvidenceUri = arguments[6];
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return _this2.loadContract();

            case 2:
              _context2.prev = 2;
              return _context2.abrupt('return', _this2.contractInstance.createTransaction(arbitratorAddress, timeout, seller, arbitratorExtraData, metaEvidenceUri, {
                from: account,
                value: value,
                gas: process.env.GAS || undefined
              }));

            case 6:
              _context2.prev = 6;
              _context2.t0 = _context2['catch'](2);

              console.error(_context2.t0);
              throw new Error(errorConstants.UNABLE_TO_CREATE_TRANSACTION);

            case 10:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2, [[2, 6]]);
    }));

    return function (_x3, _x4, _x5, _x6) {
      return _ref2.apply(this, arguments);
    };
  }();

  this.pay = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(account, arbitrableTransactionId, amount) {
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return _this2.loadContract();

            case 2:
              _context3.prev = 2;
              return _context3.abrupt('return', _this2.contractInstance.pay(arbitrableTransactionId, _this2._Web3Wrapper.toWei(amount, 'ether'), {
                from: account,
                value: 0
              }));

            case 6:
              _context3.prev = 6;
              _context3.t0 = _context3['catch'](2);

              console.error(_context3.t0);
              throw new Error(errorConstants.UNABLE_TO_PAY_SELLER);

            case 10:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this2, [[2, 6]]);
    }));

    return function (_x9, _x10, _x11) {
      return _ref3.apply(this, arguments);
    };
  }();

  this.reimburse = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(account, arbitrableTransactionId, amount) {
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return _this2.loadContract();

            case 2:
              _context4.prev = 2;
              return _context4.abrupt('return', _this2.contractInstance.reimburse(arbitrableTransactionId, _this2._Web3Wrapper.toWei(amount, 'ether'), {
                from: account,
                value: 0
              }));

            case 6:
              _context4.prev = 6;
              _context4.t0 = _context4['catch'](2);

              console.error(_context4.t0);
              throw new Error(errorConstants.UNABLE_TO_REIMBURSE_BUYER);

            case 10:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, _this2, [[2, 6]]);
    }));

    return function (_x12, _x13, _x14) {
      return _ref4.apply(this, arguments);
    };
  }();

  this.payArbitrationFeeByBuyer = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(account, arbitrableTransactionId, arbitrationCost) {
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return _this2.loadContract();

            case 2:
              _context5.prev = 2;
              return _context5.abrupt('return', _this2.contractInstance.payArbitrationFeeByBuyer(arbitrableTransactionId, {
                from: account,
                value: arbitrationCost
              }));

            case 6:
              _context5.prev = 6;
              _context5.t0 = _context5['catch'](2);

              console.error(_context5.t0);
              throw new Error(errorConstants.UNABLE_TO_PAY_ARBITRATION_FEE);

            case 10:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this2, [[2, 6]]);
    }));

    return function (_x15, _x16, _x17) {
      return _ref5.apply(this, arguments);
    };
  }();

  this.payArbitrationFeeBySeller = function () {
    var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(account, arbitrableTransactionId, arbitrationCost) {
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return _this2.loadContract();

            case 2:
              _context6.prev = 2;
              return _context6.abrupt('return', _this2.contractInstance.payArbitrationFeeBySeller(arbitrableTransactionId, {
                from: account,
                value: arbitrationCost,
                gas: process.env.GAS || undefined
              }));

            case 6:
              _context6.prev = 6;
              _context6.t0 = _context6['catch'](2);

              console.error(_context6.t0);
              throw new Error(errorConstants.UNABLE_TO_PAY_ARBITRATION_FEE);

            case 10:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, _this2, [[2, 6]]);
    }));

    return function (_x18, _x19, _x20) {
      return _ref6.apply(this, arguments);
    };
  }();

  this.submitEvidence = function () {
    var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(account, arbitrableTransactionId, url) {
      var txHashObj;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return _this2.loadContract();

            case 2:
              _context7.next = 4;
              return _this2.contractInstance.submitEvidence(arbitrableTransactionId, url, {
                from: account,
                value: 0
              });

            case 4:
              txHashObj = _context7.sent;
              return _context7.abrupt('return', txHashObj.tx);

            case 6:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, _this2);
    }));

    return function (_x21, _x22, _x23) {
      return _ref7.apply(this, arguments);
    };
  }();

  this.callTimeOutBuyer = function () {
    var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(account, arbitrableTransactionId) {
      var transactionArbitrableData, status, timeout, lastInteraction;
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.next = 2;
              return _this2.loadContract();

            case 2:
              _context8.next = 4;
              return _this2.getData(arbitrableTransactionId);

            case 4:
              transactionArbitrableData = _context8.sent;
              status = transactionArbitrableData.status;
              timeout = transactionArbitrableData.timeout;
              lastInteraction = transactionArbitrableData.lastInteraction;

              if (!(status !== contractConstants.STATUS.WAITING_SELLER)) {
                _context8.next = 12;
                break;
              }

              throw new Error(errorConstants.CONTRACT_IS_NOT_WAITING_ON_OTHER_PARTY);

            case 12:
              if (!((0, _trunc2.default)(Date.now() / 1000) <= lastInteraction + timeout)) {
                _context8.next = 14;
                break;
              }

              throw new Error(errorConstants.TIMEOUT_NOT_REACHED);

            case 14:
              _context8.prev = 14;
              return _context8.abrupt('return', _this2.contractInstance.timeOutByBuyer(arbitrableTransactionId, {
                from: account,
                value: 0
              }));

            case 18:
              _context8.prev = 18;
              _context8.t0 = _context8['catch'](14);

              console.error(_context8.t0);
              throw new Error(errorConstants.UNABLE_TO_CALL_TIMEOUT);

            case 22:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, _this2, [[14, 18]]);
    }));

    return function (_x24, _x25) {
      return _ref8.apply(this, arguments);
    };
  }();

  this.callTimeOutSeller = function () {
    var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(account, arbitrableTransactionId) {
      var status, timeout, lastInteraction;
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return _this2.loadContract();

            case 2:
              _context9.next = 4;
              return _this2.contractInstance.status();

            case 4:
              status = _context9.sent;
              _context9.next = 7;
              return _this2.contractInstance.timeout();

            case 7:
              timeout = _context9.sent;
              _context9.next = 10;
              return _this2.contractInstance.lastInteraction();

            case 10:
              lastInteraction = _context9.sent;

              if (!(status !== contractConstants.STATUS.WAITING_BUYER)) {
                _context9.next = 15;
                break;
              }

              throw new Error(errorConstants.CONTRACT_IS_NOT_WAITING_ON_OTHER_PARTY);

            case 15:
              if (!(Date.now() >= lastInteraction + timeout)) {
                _context9.next = 17;
                break;
              }

              throw new Error(errorConstants.TIMEOUT_NOT_REACHED);

            case 17:
              _context9.prev = 17;
              return _context9.abrupt('return', _this2.contractInstance.timeOutBySeller(arbitrableTransactionId, {
                from: account,
                value: 0
              }));

            case 21:
              _context9.prev = 21;
              _context9.t0 = _context9['catch'](17);

              console.error(_context9.t0);
              throw new Error(errorConstants.UNABLE_TO_CALL_TIMEOUT);

            case 25:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, _this2, [[17, 21]]);
    }));

    return function (_x26, _x27) {
      return _ref9.apply(this, arguments);
    };
  }();

  this.appeal = function () {
    var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(account, arbitrableTransactionId) {
      var extraData = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0x0;
      var appealCost = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0.3;
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return _this2.loadContract();

            case 2:
              _context10.prev = 2;
              return _context10.abrupt('return', _this2.contractInstance.appeal(arbitrableTransactionId, extraData, {
                from: account,
                value: appealCost
              }));

            case 6:
              _context10.prev = 6;
              _context10.t0 = _context10['catch'](2);

              console.error(_context10.t0);
              throw new Error(errorConstants.UNABLE_TO_RAISE_AN_APPEAL);

            case 10:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, _this2, [[2, 6]]);
    }));

    return function (_x28, _x29) {
      return _ref10.apply(this, arguments);
    };
  }();

  this.setArbitrableTransactionId = function (arbitrableTransactionId) {
    return _this2.arbitrableTransactionId = arbitrableTransactionId;
  };

  this.getParties = function () {
    var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(arbitrableTransactionId) {
      var arbitrableTransaction;
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              _context11.next = 2;
              return _this2.loadContract();

            case 2:
              _context11.next = 4;
              return _this2.contractInstance.transactions(arbitrableTransactionId);

            case 4:
              arbitrableTransaction = _context11.sent;
              return _context11.abrupt('return', {
                seller: arbitrableTransaction[0],
                buyer: arbitrableTransaction[1]
              });

            case 6:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, _this2);
    }));

    return function (_x32) {
      return _ref11.apply(this, arguments);
    };
  }();

  this.getData = function () {
    var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(arbitrableTransactionId) {
      var arbitrableTransaction;
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _context12.next = 2;
              return _this2.loadContract();

            case 2:
              _context12.next = 4;
              return _this2.contractInstance.transactions(arbitrableTransactionId);

            case 4:
              arbitrableTransaction = _context12.sent;
              _context12.t0 = arbitrableTransaction[0];
              _context12.t1 = arbitrableTransaction[1];
              _context12.t2 = _this2._Web3Wrapper.fromWei(arbitrableTransaction[2].toNumber(), 'ether');
              _context12.t3 = arbitrableTransaction[3].toNumber();
              _context12.t4 = arbitrableTransaction[4].toNumber();
              _context12.t5 = arbitrableTransaction[5];
              _context12.t6 = arbitrableTransaction[6];
              _context12.t7 = _this2._Web3Wrapper.fromWei(arbitrableTransaction[7], 'ether');
              _context12.t8 = _this2._Web3Wrapper.fromWei(arbitrableTransaction[8], 'ether');
              _context12.t9 = arbitrableTransaction[9].toNumber();
              _context12.t10 = arbitrableTransaction[10].toNumber();
              _context12.next = 18;
              return _this2.getMetaEvidence(arbitrableTransactionId);

            case 18:
              _context12.t11 = _context12.sent;
              _context12.next = 21;
              return _this2.getEvidence(arbitrableTransaction[5], arbitrableTransaction[4].toNumber());

            case 21:
              _context12.t12 = _context12.sent;
              return _context12.abrupt('return', {
                seller: _context12.t0,
                buyer: _context12.t1,
                amount: _context12.t2,
                timeout: _context12.t3,
                disputeId: _context12.t4,
                arbitrator: _context12.t5,
                arbitratorExtraData: _context12.t6,
                sellerFee: _context12.t7,
                buyerFee: _context12.t8,
                lastInteraction: _context12.t9,
                status: _context12.t10,
                metaEvidence: _context12.t11,
                evidence: _context12.t12
              });

            case 23:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, _this2);
    }));

    return function (_x33) {
      return _ref12.apply(this, arguments);
    };
  }();
};

exports.default = MultipleArbitrableTransaction;
//# sourceMappingURL=MultipleArbitrableTransaction.js.map