'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _KlerosPOC = require('kleros/build/contracts/KlerosPOC');

var _KlerosPOC2 = _interopRequireDefault(_KlerosPOC);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _eth = require('../../../constants/eth');

var ethConstants = _interopRequireWildcard(_eth);

var _deployContractAsync = require('../../../utils/deployContractAsync');

var _deployContractAsync2 = _interopRequireDefault(_deployContractAsync);

var _Kleros2 = require('./Kleros');

var _Kleros3 = _interopRequireDefault(_Kleros2);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides interaction with a KlerosPOC contract on the blockchain.
 */
var KlerosPOC = function (_Kleros) {
  (0, _inherits3.default)(KlerosPOC, _Kleros);

  /**
   * Create new KlerosPOC Implementation.
   * @param {object} web3Provider - web3 instance.
   * @param {string} contractAddress - Address of the KlerosPOC contract.
   */
  function KlerosPOC(web3Provider, contractAddress) {
    (0, _classCallCheck3.default)(this, KlerosPOC);

    var _this = (0, _possibleConstructorReturn3.default)(this, (KlerosPOC.__proto__ || (0, _getPrototypeOf2.default)(KlerosPOC)).call(this, web3Provider, contractAddress, _KlerosPOC2.default));

    _initialiseProps.call(_this);

    return _this;
  }

  /**
   * STATIC: Deploy a KlerosPOC contract on the blockchain.
   * @param {string} rngAddress address of random number generator contract
   * @param {string} pnkAddress address of pinakion contract
   * @param {number[]} timesPerPeriod array of 5 ints indicating the time limit for each period of contract
   * @param {string} account address of user
   * @param {number} value amout of eth to send to contract
   * @param {object} web3Provider web3 provider object NOTE: NOT Kleros Web3Wrapper
   * @returns {object} truffle-contract Object | err The contract object or error deploy
   */


  /**
   * Purchase PNK.
   * @param {string} amount - The amount of pinakion to buy in wei.
   * @param {string} account - The address of the user.
   * @returns {object} - The result transaction object.
   */


  return KlerosPOC;
}(_Kleros3.default);

KlerosPOC.deploy = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(rngAddress, pnkAddress) {
    var timesPerPeriod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [300, 0, 300, 300, 300];
    var account = arguments[3];
    var value = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : ethConstants.TRANSACTION.VALUE;
    var web3Provider = arguments[5];
    var contractDeployed;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _deployContractAsync2.default)(account, value, _KlerosPOC2.default, web3Provider, pnkAddress, rngAddress, timesPerPeriod);

          case 2:
            contractDeployed = _context.sent;
            return _context.abrupt('return', contractDeployed);

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.buyPNK = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(amount, account) {
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return _this2.loadContract();

            case 2:
              return _context2.abrupt('return', _this2.contractInstance.buyPinakion({
                from: account,
                value: amount,
                gas: process.env.GAS || undefined
              }));

            case 3:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2);
    }));

    return function (_x5, _x6) {
      return _ref2.apply(this, arguments);
    };
  }();
};

exports.default = KlerosPOC;
//# sourceMappingURL=KlerosPOC.js.map