'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _arbitrator = require('../constants/arbitrator');

var arbitratorConstants = _interopRequireWildcard(_arbitrator);

var _dispute = require('../constants/dispute');

var disputeConstants = _interopRequireWildcard(_dispute);

var _isRequired = require('../utils/isRequired');

var _isRequired2 = _interopRequireDefault(_isRequired);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Disputes API. Provides cross arbitrator and arbitrable contracts functionality.
 * Requires Store Provider to be set.
 */
var Disputes = function Disputes() {
  var arbitratorInstance = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _isRequired2.default)('arbitratorInstance');
  var arbitrableInstance = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _isRequired2.default)('arbitrableInstance');
  var storeProviderInstance = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (0, _isRequired2.default)('storeProviderInstance');
  (0, _classCallCheck3.default)(this, Disputes);

  _initialiseProps.call(this);

  this._ArbitratorInstance = arbitratorInstance;
  this._ArbitrableInstance = arbitrableInstance;
  this._StoreProviderInstance = storeProviderInstance;
  this.disputeCache = {};
}
/**
 * Set arbitrator instance.
 * @param {object} arbitratorInstance - instance of an arbitrator contract.
 */

/**
 * Set arbitrable instance.
 * @param {object} arbitrableInstance - instance of an arbitrable contract.
 */

/**
 * Set store provider instance.
 * @param {object} storeProviderInstance - instance of store provider wrapper.
 */


// **************************** //
// *         Events           * //
// **************************** //

/**
 * Method to register all dispute handlers to an EventListener.
 * @param {string} account - The address of the user.
 * @param {object} eventListener - The EventListener instance. See utils/EventListener.js.
 */


/**
 * Event listener handler that stores dispute in store upon creation
 * @param {string} event - The event log.
 * @param {string} account - Account of user to update store data for
 */


// **************************** //
// *        Internal          * //
// **************************** //

/**
 * Add new data to the cache
 * @param {number} disputeID - The index of the dispute. Used as the key in cache
 * @param {object} newCacheData - Freeform data to cache. Will overwrite data with the same keys.
 */


/**
 * Get the block at which a dispute was created. Used to find timestamps for dispute.
 * The start block is cached after it has been found once as it will never change.
 * @param {number} disputeID - The index of the dispute.
 * @param {string} account - The address of the user.
 * @returns {number} The block number that the dispute was created.
 */


// **************************** //
// *          Public          * //
// **************************** //
/**
 * Fetch the shared dispute data from the store.
 * @param {string} account - The users account.
 * @param {string} disputeID - The index of the dispute.
 * @returns {Promise} The dispute data in the store.
 */


/**
 * Get the dispute deadline for the appeal.
 * @param {number} disputeID - The index of the dispute.
 * @param {number} appeal - The appeal number. 0 if there have been no appeals.
 * @returns {number} timestamp of the appeal
 */


/**
 * Get the timestamp on when the dispute's ruling was finalized.
 * @param {number} disputeID - The index of the dispute.
 * @param {number} appeal - The appeal number. 0 if there have been no appeals.
 * @returns {number} timestamp of the appeal
 */


/**
 * Get the timestamp on when the dispute's appeal was created
 * @param {number} disputeID - The index of the dispute.
 * @param {string} account - The users address.
 * @param {number} appeal - The appeal number. 0 if there have been no appeals.
 * @returns {number} timestamp of the appeal
 */


/**
 * Get data for a dispute. This method provides data from the store as well as both
 * arbitrator and arbitrable contracts. Used to get all relevant data on a dispute.
 * @param {number} disputeID - The dispute's ID.
 * @param {string} account - The juror's address.
 * @returns {object} - Data object for the dispute that uses data from the contract and the store.
 */
;

var _initialiseProps = function _initialiseProps() {
  var _this = this;

  this.setArbitratorInstance = function (arbitratorInstance) {
    _this._ArbitratorInstance = arbitratorInstance;
  };

  this.setArbitrableInstance = function (arbitrableInstance) {
    _this._ArbitrableInstance = arbitrableInstance;
  };

  this.setStoreProviderInstance = function (storeProviderInstance) {
    _this._StoreProviderInstance = storeProviderInstance;
  };

  this.registerStoreUpdateEventListeners = function () {
    var account = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _isRequired2.default)('account');
    var eventListener = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _isRequired2.default)('eventListener');

    var eventHandlerMap = {
      DisputeCreation: [_this._storeNewDisputeHandler]
    };

    var _loop = function _loop(event) {
      if (eventHandlerMap.hasOwnProperty(event)) {
        eventHandlerMap[event].forEach(function (handler) {
          eventListener.addEventHandler(_this._ArbitratorInstance, event, function (args) {
            return handler(args, account);
          });
        });
      }
    };

    for (var event in eventHandlerMap) {
      _loop(event);
    }
  };

  this._storeNewDisputeHandler = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(event, account) {
      var disputeID, arbitratorAddress, disputeData, arbitrableContractData;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              // There is no need to handle this event if we are not using the store
              disputeID = event.args._disputeID.toNumber();
              arbitratorAddress = _this._ArbitratorInstance.getContractAddress();
              // arbitrator data

              _context.next = 4;
              return _this._ArbitratorInstance.getDispute(disputeID);

            case 4:
              disputeData = _context.sent;
              _context.next = 7;
              return _this._ArbitrableInstance.setContractInstance(disputeData.arbitrableContractAddress);

            case 7:
              _context.next = 9;
              return _this._ArbitrableInstance.getData(account);

            case 9:
              arbitrableContractData = _context.sent;

              if (!(account === arbitrableContractData.partyA || account === arbitrableContractData.partyB)) {
                _context.next = 13;
                break;
              }

              _context.next = 13;
              return _this._StoreProviderInstance.updateDisputeProfile(account, arbitratorAddress, disputeID, {
                contractAddress: disputeData.arbitrableContractAddress,
                partyA: arbitrableContractData.partyA,
                partyB: arbitrableContractData.partyB,
                blockNumber: event.blockNumber
              });

            case 13:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x6, _x7) {
      return _ref.apply(this, arguments);
    };
  }();

  this._updateDisputeCache = function (disputeID) {
    var newCacheData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _this.disputeCache[disputeID] = (0, _extends3.default)({}, _this.disputeCache[disputeID], newCacheData);
  };

  this._getDisputeStartBlock = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(disputeID, account) {
      var cachedDispute, arbitratorAddress, blockNumber, userData, disputeCreationEvent;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              cachedDispute = _this.disputeCache[disputeID];

              if (!(cachedDispute && cachedDispute.startBlock)) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt('return', cachedDispute.startBlock);

            case 3:
              arbitratorAddress = _this._ArbitratorInstance.getContractAddress();
              blockNumber = void 0;
              _context2.prev = 5;
              _context2.next = 8;
              return _this._StoreProviderInstance.getDispute(account, arbitratorAddress, disputeID);

            case 8:
              userData = _context2.sent;

              blockNumber = userData.blockNumber;
              // eslint-disable-next-line no-unused-vars
              _context2.next = 14;
              break;

            case 12:
              _context2.prev = 12;
              _context2.t0 = _context2['catch'](5);

            case 14:
              if (blockNumber) {
                _context2.next = 19;
                break;
              }

              _context2.next = 17;
              return _this._ArbitratorInstance.getDisputeCreationEvent(disputeID);

            case 17:
              disputeCreationEvent = _context2.sent;

              if (disputeCreationEvent) {
                blockNumber = disputeCreationEvent.blockNumber;
              }

            case 19:

              // cache start block for dispute
              _this._updateDisputeCache(disputeID, { startBlock: blockNumber });
              return _context2.abrupt('return', blockNumber);

            case 21:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this, [[5, 12]]);
    }));

    return function (_x9, _x10) {
      return _ref2.apply(this, arguments);
    };
  }();

  this.getDisputeFromStore = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(account, disputeID) {
      var arbitratorAddress;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              arbitratorAddress = _this._ArbitratorInstance.getContractAddress();
              return _context3.abrupt('return', _this._StoreProviderInstance.getDispute(account, arbitratorAddress, disputeID));

            case 2:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this);
    }));

    return function (_x11, _x12) {
      return _ref3.apply(this, arguments);
    };
  }();

  this.getDisputeDeadline = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(disputeID) {
      var appeal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var cachedDispute, dispute, deadlineTimestamp, currentDeadlines;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              cachedDispute = _this.disputeCache[disputeID] || {};

              if (!(cachedDispute.appealDeadlines && cachedDispute.appealDeadlines[appeal])) {
                _context4.next = 3;
                break;
              }

              return _context4.abrupt('return', cachedDispute.appealDeadlines[appeal]);

            case 3:
              _context4.next = 5;
              return _this._ArbitratorInstance.getDispute(disputeID);

            case 5:
              dispute = _context4.sent;
              _context4.next = 8;
              return _this._ArbitratorInstance.getDisputeDeadlineTimestamp(dispute.firstSession + appeal);

            case 8:
              deadlineTimestamp = _context4.sent;


              if (deadlineTimestamp) {
                currentDeadlines = cachedDispute.appealDeadlines || [];

                currentDeadlines[appeal] = deadlineTimestamp;
                // cache the deadline for the appeal
                _this._updateDisputeCache(disputeID, {
                  appealDeadlines: currentDeadlines
                });
              }

              return _context4.abrupt('return', deadlineTimestamp);

            case 11:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, _this);
    }));

    return function (_x13) {
      return _ref4.apply(this, arguments);
    };
  }();

  this.getAppealRuledAt = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(disputeID) {
      var appeal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var cachedDispute, dispute, appealRuledAtTimestamp, currentRuledAt;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              cachedDispute = _this.disputeCache[disputeID] || {};

              if (!(cachedDispute.appealRuledAt && cachedDispute.appealRuledAt[appeal])) {
                _context5.next = 3;
                break;
              }

              return _context5.abrupt('return', cachedDispute.appealRuledAt[appeal]);

            case 3:
              _context5.next = 5;
              return _this._ArbitratorInstance.getDispute(disputeID);

            case 5:
              dispute = _context5.sent;
              _context5.next = 8;
              return _this._ArbitratorInstance.getAppealRuledAtTimestamp(dispute.firstSession + appeal);

            case 8:
              appealRuledAtTimestamp = _context5.sent;


              // cache the deadline for the appeal
              if (appealRuledAtTimestamp) {
                currentRuledAt = cachedDispute.appealRuledAt || [];

                currentRuledAt[appeal] = appealRuledAtTimestamp;
                _this._updateDisputeCache(disputeID, {
                  appealRuledAt: currentRuledAt
                });
              }

              return _context5.abrupt('return', appealRuledAtTimestamp);

            case 11:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this);
    }));

    return function (_x15) {
      return _ref5.apply(this, arguments);
    };
  }();

  this.getAppealCreatedAt = function () {
    var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(disputeID, account) {
      var appeal = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var cachedDispute, dispute, appealCreatedAtTimestamp, creationBlock, timestampSeconds, currentCreatedAt;
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              cachedDispute = _this.disputeCache[disputeID] || {};

              if (!(cachedDispute.appealCreatedAt && cachedDispute.appealCreatedAt[appeal])) {
                _context6.next = 3;
                break;
              }

              return _context6.abrupt('return', cachedDispute.appealCreatedAt[appeal]);

            case 3:
              _context6.next = 5;
              return _this._ArbitratorInstance.getDispute(disputeID);

            case 5:
              dispute = _context6.sent;
              appealCreatedAtTimestamp = null;

              if (!(appeal === 0)) {
                _context6.next = 18;
                break;
              }

              _context6.next = 10;
              return _this._getDisputeStartBlock(disputeID, account);

            case 10:
              creationBlock = _context6.sent;

              if (!creationBlock) {
                _context6.next = 16;
                break;
              }

              _context6.next = 14;
              return _this._ArbitratorInstance._getTimestampForBlock(creationBlock);

            case 14:
              timestampSeconds = _context6.sent;


              appealCreatedAtTimestamp = timestampSeconds * 1000;

            case 16:
              _context6.next = 22;
              break;

            case 18:
              _context6.next = 20;
              return _this._ArbitratorInstance.getAppealCreationTimestamp(dispute.firstSession + (appeal - 1) // appeal was created during previous session
              );

            case 20:
              appealCreatedAtTimestamp = _context6.sent;


              // cache the deadline for the appeal
              if (appealCreatedAtTimestamp) {
                currentCreatedAt = cachedDispute.appealCreatedAt || [];

                currentCreatedAt[appeal] = appealCreatedAtTimestamp;
                _this._updateDisputeCache(disputeID, {
                  appealCreatedAt: currentCreatedAt
                });
              }

            case 22:
              return _context6.abrupt('return', appealCreatedAtTimestamp);

            case 23:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, _this);
    }));

    return function (_x17, _x18) {
      return _ref6.apply(this, arguments);
    };
  }();

  this.getDataForDispute = function () {
    var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(disputeID, account) {
      var arbitratorAddress, _ref8, _ref9, dispute, period, session, arbitrableContractAddress, _ref10, _ref11, metaEvidence, evidence, parties, appealDraws, userData, netPNK, lastSession, appealJuror, appealRulings, appeal, isLastAppeal, draws, canRule, canRepartition, canExecute, ruling, rulingPromises, _ref12, _ref13, jurorRuling, appealCreatedAt, appealDeadline, appealRuledAt;

      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              arbitratorAddress = _this._ArbitratorInstance.getContractAddress();
              // Get dispute data from contract. Also get the current session and period.

              _context7.next = 3;
              return _promise2.default.all([_this._ArbitratorInstance.getDispute(disputeID, true), _this._ArbitratorInstance.getPeriod(), _this._ArbitratorInstance.getSession()]);

            case 3:
              _ref8 = _context7.sent;
              _ref9 = (0, _slicedToArray3.default)(_ref8, 3);
              dispute = _ref9[0];
              period = _ref9[1];
              session = _ref9[2];


              // Get arbitrable contract data and evidence
              arbitrableContractAddress = dispute.arbitrableContractAddress;
              _context7.next = 11;
              return _this._ArbitrableInstance.setContractInstance(arbitrableContractAddress);

            case 11:
              _context7.next = 13;
              return _promise2.default.all([_this._ArbitrableInstance.getMetaEvidence(), _this._ArbitrableInstance.getEvidence(), _this._ArbitrableInstance.getParties()]);

            case 13:
              _ref10 = _context7.sent;
              _ref11 = (0, _slicedToArray3.default)(_ref10, 3);
              metaEvidence = _ref11[0];
              evidence = _ref11[1];
              parties = _ref11[2];


              // Get dispute data from the store
              appealDraws = [];

              // get draws if they have been added to store.

              _context7.prev = 19;
              _context7.next = 22;
              return _this._StoreProviderInstance.getDispute(account, arbitratorAddress, disputeID);

            case 22:
              userData = _context7.sent;

              if (userData.appealDraws) appealDraws = userData.appealDraws || [];
              // eslint-disable-next-line no-unused-vars
              _context7.next = 29;
              break;

            case 26:
              _context7.prev = 26;
              _context7.t0 = _context7['catch'](19);

              // Dispute exists on chain but not in store. We have lost draws for past disputes.
              console.error('Dispute does not exist in store.');

            case 29:
              _context7.next = 31;
              return _this._ArbitratorInstance.getNetTokensForDispute(disputeID, account);

            case 31:
              netPNK = _context7.sent;


              // Build juror info and ruling arrays, indexed by appeal number
              lastSession = dispute.firstSession + dispute.numberOfAppeals;
              appealJuror = [];
              appealRulings = [];
              appeal = 0;

            case 36:
              if (!(appeal <= dispute.numberOfAppeals)) {
                _context7.next = 71;
                break;
              }

              isLastAppeal = dispute.firstSession + appeal === lastSession;
              // Get appeal data

              draws = appealDraws[appeal] || [];
              canRule = false;
              canRepartition = false;
              canExecute = false;
              ruling = void 0;
              rulingPromises = [_this._ArbitratorInstance.currentRulingForDispute(disputeID)];

              // Extra info for the last appeal

              if (isLastAppeal) {
                if (draws.length > 0) rulingPromises.push(_this._ArbitratorInstance.canRuleDispute(disputeID, draws, account));

                if (session && period) canRepartition = lastSession <= session && // Not appealed to the next session
                period === arbitratorConstants.PERIOD.EXECUTE && // Executable period
                dispute.state === disputeConstants.STATE.OPEN; // Open dispute
                canExecute = dispute.state === disputeConstants.STATE.EXECUTABLE; // Executable state
              }

              // Wait for parallel requests to complete
              ;_context7.next = 48;
              return _promise2.default.all(rulingPromises);

            case 48:
              _ref12 = _context7.sent;
              _ref13 = (0, _slicedToArray3.default)(_ref12, 2);
              ruling = _ref13[0];
              canRule = _ref13[1];
              jurorRuling = null;
              // if can't rule that means they already did or they missed it

              if (canRule) {
                _context7.next = 57;
                break;
              }

              _context7.next = 56;
              return _this._ArbitratorInstance.getVoteForJuror(dispute.disputeID, appeal, account);

            case 56:
              jurorRuling = _context7.sent;

            case 57:
              _context7.next = 59;
              return _this.getAppealCreatedAt(dispute.disputeID, account, appeal);

            case 59:
              appealCreatedAt = _context7.sent;
              _context7.next = 62;
              return _this.getDisputeDeadline(dispute.disputeID, appeal);

            case 62:
              appealDeadline = _context7.sent;
              _context7.next = 65;
              return _this.getAppealRuledAt(dispute.disputeID, appeal);

            case 65:
              appealRuledAt = _context7.sent;


              appealJuror[appeal] = {
                createdAt: appealCreatedAt,
                fee: dispute.arbitrationFeePerJuror.mul(draws.length),
                draws: draws,
                jurorRuling: jurorRuling,
                canRule: canRule
              };
              appealRulings[appeal] = {
                voteCounter: dispute.voteCounters[appeal],
                deadline: appealDeadline,
                ruledAt: appealRuledAt,
                ruling: ruling,
                canRepartition: canRepartition,
                canExecute: canExecute
              };

            case 68:
              appeal++;
              _context7.next = 36;
              break;

            case 71:
              return _context7.abrupt('return', {
                // Arbitrable Contract Data
                arbitrableContractAddress: arbitrableContractAddress,
                arbitratorAddress: arbitratorAddress,
                parties: parties,
                evidence: evidence,
                metaEvidence: metaEvidence,

                // Dispute Data
                disputeID: disputeID,
                firstSession: dispute.firstSession,
                lastSession: lastSession,
                numberOfAppeals: dispute.numberOfAppeals,
                disputeState: dispute.state,
                disputeStatus: dispute.status,
                appealJuror: appealJuror,
                appealRulings: appealRulings,
                netPNK: netPNK
              });

            case 72:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, _this, [[19, 26]]);
    }));

    return function (_x20, _x21) {
      return _ref7.apply(this, arguments);
    };
  }();
};

exports.default = Disputes;
//# sourceMappingURL=Disputes.js.map